<?php
/*
Copyright (c) 2006 Yasuo Ohgaki All Rights Reserved

This file is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.
 */

// 強制ブラウズ対策
if (!defined('CONFIG')) {
	error_log(
		'直接アクセスを検知しました。 ファイル: '. __FILE__.' 行:'.__LINE__.
		' IP: '. $_SERVER['REMOTE_ADDR'] .' AGENT: '. $_SERVER['HTTP_USER_AGENT']);
	die('このファイルは直接参照できません。');// 確実に実行を終了する
}

class BBS_Auth {
	private $db;

	public function __construct($db)
	{
		$this->db = $db; // BBS_DB_Main
	}

	// 認証済みか確認
	public function CheckAuth() {
		// 有効期限切れ、セッションIDの再作成はセッション開始時に処理される
		if (empty($_SESSION)) {
			// セッションが開始されていない
			trigger_error('セッションが開始されていません。', E_USER_ERROR);
			exit; // エラーハンドラで終了するが、このようにexitを付けるとより明確
		}
		// 自動ログインクッキーあり。自動ログインを試す
		if (empty($_SESSION['AUTH']) && !empty($_COOKIE['AUTOLOGIN'])) {
			if (validate_sha1($_COOKIE['AUTOLOGIN'])) {
				$this->AutoLogin($_COOKIE['AUTOLOGIN']);
			}
		}

		// ログインが必要
		if (empty($_SESSION['AUTH'])) {
			header('Location: http://'. $_SERVER['HTTP_HOST'] . URI_PATH .'login/');
			ob_get_clean();exit; // リダイレクトはフルパス。他の出力はしないで終了する。
		}
	}

	public function Login($username, $password) {
		if ($this->db->GetUser($username, $password)) {
			// 新しいセッションIDを使用。trueを渡して古いセッションを削除。
			session_regenerate_id(true);   // セッションIDの固定化を防止。
			$_SESSION['AUTH'] = true;
			if (!empty($_POST['autologin'])) {
				$this->SetAutoLogin($username);
			}
			return true;
		} else {
			return false;
		}
	}

	public function Logoff() {
		$status = !empty($_SESSION['AUTH']);
		$this->UnsetAutoLogin(); // 自動ログインを無効化
		session_destroy(); //セッションデータを削除。
		// 念には念を入れて認証フラグを削除（session_destory()で削除されているはず）
		unset($_SESSION['AUTH']);
		return $status;
	}

	// 自動ログイン処理
	public function AutoLogin($key) {
		$username = $this->db->GetAutoLogin($key);
		if ($username) {
			// 自動ログイン処理
			if ($this->db->GetUser($username, null, true)) {
				// 新しいセッションIDを使用。trueを渡して古いセッションを削除。
				session_regenerate_id(true);   // セッションIDの固定化を防止。
				$_SESSION['AUTH'] = true;
			}
			// 自動ログインクッキーの再設定
			$this->SetAutoLogin($username);
			return true;
		}
		return false;
	}

	// 自動ログインクッキーの設定
	public function SetAutoLogin($username) {
		$key = $this->db->SetAutoLogin($username);
		// クッキーを送信するPATHを限定、HttpOnlyを設定して安全性を向上
		if (!setcookie('AUTOLOGIN', $key, time()+3600*24*15, URI_PATH.'login/', '', false, true)) {
			trigger_error('自動ログインクッキーの設定に失敗しました。');
		}
	}

	// 自動ログインクッキーの削除
	public function UnsetAutoLogin() {
		if (!setcookie('AUTOLOGIN', '', 0, URI_PATH.'login/')) {
			trigger_error('自動ログインクッキーの削除に失敗しました。');
		}
		if (!empty($_COOKIE['AUTOLOGIN']) && !$this->db->UnsetAutoLogin($_COOKIE['AUTOLOGIN'])) {
			trigger_error('自動ログインキーの削除に失敗しました。');
		}
		return true;
	}
} // BBS_Auth

