<?php
/*
Copyright (c) 2006 Yasuo Ohgaki All Rights Reserved

This file is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.
 */

// 強制ブラウズ対策
if (!defined('CONFIG')) {
	error_log(
		'直接アクセスを検知しました。 ファイル: '. __FILE__.' 行:'.__LINE__.
		' IP: '. $_SERVER['REMOTE_ADDR'] .' AGENT: '. $_SERVER['HTTP_USER_AGENT']);
	die('このファイルは直接参照できません。');// 確実に実行を終了する
}

// 入力確認配列のインデックスとその意味
define('V_METHOD', 0); // post, getメソッド
define('V_FUNC', 1);   // validation関数
define('V_MESSAGE', 2);// エラーメッセージ。（RETURNの場合）
define('V_PARAM',3);   // 引数

class BBS {
	// データベースオブジェクト
	private $db;
	
	// 認証オブジェクト
	private $auth;

	// BBSアプリケーションのモジュール
	private $modules =
		array(
			'login'         => 'bbs_login/index.php',
			'thread'        =>'thread.php',
			'thread_create' => 'thread_create.php',
		);

	// 各モジュールのアクション
	private $actions =
		array(
			'login' =>
			array(
				'default'=>'login_form',
				'login_form'=>'login_form',
				'logoff'=>'logoff',
				'submit_btn'=>'login',
			),
			'thread'=>
			array(
				'default'=>'thread_list',
				'thread_list'=>'thread_list',
				'thread_view'=>'thread_view',
			),
			'thread_create'=>
			array(
				'default' => 'thread_edit',
				'thread_edit' => 'thread_edit',
				'submit_btn'=>'thread_submit',
			),
		);

	// 各アクションの入力（ブラウザからの入力）
	private $inputs =
		array(
			'@login:login_form' =>
			array(
				'validations' =>
				array(
					// バリデーションもエラーも、必要ない場合も配列は定義
					array(),
				),
				'errors' =>
				array(
					array(),
				),
			),

			'@login:login' =>
			array(
				'validations' =>
				array(
					'username' =>
					array(
						//    V_METHOD, V_FUNC, V_MESSAGE, V_PARAM
						array('post', 'str'),
						array('post', 'str_length', '', 0, 250),
					),
					'password' =>
					array(
						array('post', 'str'),
						array('post', 'str_length', '', 0, 250),
					),
					'autologin' =>
					array(
						array('post', 'int_range', '', 0, 1),
					),
				),
				'errors' =>
				array(
					// 全ての記号、英数字。ただし、環境によっては入力しずらい文字もあるので\、~などは避ける方が良い
					'username' =>
					array(
						array('post', 'str_ereg',   'ユーザ名の入力形式が不正です', '[^[!-~]]'),
						array('post', 'str_length', 'ユーザ名の長さは5から64文字です', 5, 64),
					),
					'password' =>
					array(
						array('post', 'str_ereg',   'パスワードの入力形式が不正です', '[^[!-~]]'),
						// password_hash()の制限に合わせておく
						array('post', 'str_length', 'パスワードの長さは5から72文字です', 5, 72),
					),
				),
			),

			'@login:logoff' =>
			array(
				'validations' =>
				array(
					// バリデーションもエラーも、必要ない場合も配列は定義
					array(),
				),
				'errors' =>
				array(
					array(),
				),
			),

			'@thread:thread_list' =>
			array(
				'validations' =>
				array(
					// バリデーションもエラーも、必要ない場合も配列は定義
					array(),
				),
				'errors' =>
				array(
					array(),
				),
			),

			'@thread_create:thread_edit' =>
			array(
				'validations' =>
				array(
					// バリデーションもエラーも、必要ない場合も配列は定義
					array(),
				),
				'errors' =>
				array(
					array(),
				),
			),

			'@thread:thread_view' =>
			array(
				'validations' =>
				array(
					'thread_id' =>
					array(
						array('get', 'int_type'),
					),
				),
				'errors' =>
				array(
					array(),
				),
			),

			'@thread_create:thread_submit' =>
			array(
				'validations' =>
				array(
					'title' =>
					array(
						array('post', 'str'),
						array('post', 'str_length', '', 0, 1000),
					),
					'message' =>
					array(
						array('post', 'str'),
						array('post', 'str_length', '', 0, 30000),
					),
					'form_id' =>
					array(
						array('post', 'str'),
						array('post', 'str_length', '', 0, 81),
					)
				),
				'errors' =>
				array(
					'title' =>
					array(
						array('post', 'str_width','タイトルが無いか長すぎます。' , 1, 60),
					),
					'message' =>
					array(
						array('post', 'str_width','メッセージが無いか長すぎます。' , 1, 1000),
					),
				),
			)
		);

	public  $module;
	public  $action;

	public function __construct($auth = true) {
		$this->db = new BBS_DB_Main;
		$this->auth = new BBS_Auth($this->db);
		if ($auth) {
			$this->auth->CheckAuth();
		}
	}

	////////////// 認証 ///////////////////////
	public function Login($username, $password) {
		return $this->auth->Login($username, $password);
	}

	public function Logoff() {
		return $this->auth->Logoff();
	}

	public function AutoLogin($key) {
		return $this->auth->AutoLogin($key);
	}

	public function DisableAutoLogin() {
		return $this->auth->UnsetAutoLogin();
	}

	////////////// 制御 ///////////////////////
	public function GetAction($module) {
		$this->modules[$module]; // typo対策
		$this->module = $module;
		if (isset($_POST['verify_btn'])) {
			// verify_btn（確認ボタン）
			$this->action = $this->actions[$module]['verify_btn'];
		}
		else if (isset($_POST['submit_btn'])) {
			// submit_btn(送信)
			$this->action = $this->actions[$module]['submit_btn'];
		}
		else {
			// $_GET['action']による設定
			if (empty($_GET['action'])) {
				// デフォルトアクション
				$this->action = $this->actions[$module]['default'];
			} else {
				$this->action = $this->actions[$module][$_GET['action']];
			}
		}
		
	}

	/////////// 入力確認 ///////////////////////
	// 本来はこの種類のメソッドはベースクラスに実装
	public function ValidateInputs() {
		global $get, $post;
		$get = $post = array();

		if (empty($this->module) || empty($this->action)) {
			trigger_error('アプリケーションエラー。');
		}

		if (count($_COOKIE) > 2) {
			// BBSアプリケーションでは2つのクッキーのみ使用。
			// 他のアプリケーションも同時に利用する場合、ここでエラーになる可能性あり。
			error_log('無用なクッキーが送信されました。 -'. serialize($_COOKIE));
		}
		
		//現在のモジュールとアクション
		$st = '@'. $this->module . ':' . $this->action;
		// 確認対象の設定
		$inputs = $this->inputs[$st];
		// $_GET/$_POSTをチェック
		$errors = array();
		unset($_POST['submit_btn']);unset($_POST['verify_btn']); // 送信ボタンを無視
		$g = $_GET;$p = $_POST; // 無用な値の送信チェック用
		unset($g['action']);    // 各モジュールで確認される

		// 入力バリデーション処理
		foreach($inputs['validations'] as $name => $prop) {
			foreach($prop as $elem) {
				// エラー発生でエラーハンドラが呼ばれスクリプト停止
				$this->Validate($name, $elem, $p, $g, $errors, true);
			}
		}

		// 入力エラー処理
		// バリデーション関数を流用
		foreach($inputs['errors'] as $name => $prop) {
			foreach($prop as $elem) {
				// エラー発生で$errorsにエラーメッセージ設定
				$this->Validate($name, $elem, $p, $g, $errors);
			}
		}

		if (count($p) || count($g)) {
			// ユーザが無用な値を送信
			trigger_error('不必要な情報が送信されました。'. print_r($p, true) .' '. print_r($g, true));
		}

		return $errors;
	}

	private function Validate($name, $prop, &$p, &$g, &$errors, $validate=false) {
		$func = 'validate_'. $prop[V_FUNC];
		$param = array_slice($prop, V_PARAM);// 関数の引数
		if (!$validate) {
			$param[] = VLD_RETURN; // 入力エラーチェックはバリデーション関数を流用しているので、エラーとせずに結果を戻す
		}
		if (!empty($prop[V_FUNC]) && isset($_REQUEST[$name])) {
			switch ($prop[V_METHOD]) {
				case 'post':
					$value = $post[$name] = $_POST[$name];
						break;
				case 'get':
					$value = $get[$name] = $_GET[$name];
					break;
				default:
					trigger_error('入力値の定義に誤りがありました。');
			}
			$param = array_merge(array($value), $param); // 確認対象引数を追加
			if ($validate) {
				// バリデーションエラーはプログラム終了なので呼ぶだけ
				call_user_func_array($func, $param);
			} else {
				// 入力エラーは実行を継続するのでエラーメッセージを保存
				if (!call_user_func_array($func, $param)) {
					$errors[] = $prop[V_MESSAGE]; // エラー
				}
			}
			if (DEBUG == 2) {
				echo ($validate ? 'VALIDATED: ' : 'CHECKED: '). $func .' '. join($param,',') .'<br />';
			}
		}
		unset($p[$name]);unset($g[$name]); // チェックした値を削除
	}
	
	////////////// スレッド ////////////////////
	public function GetThreads() {
		return $this->db->GetThreads();
	}

	public function CreateThread($title, $message) {
		$this->db->CreateThread($title, $message);
	}

	public function GetThread($thread_id) {
		return $this->db->GetThread($thread_id);
	}

	/////////////// フォームID //////////////
	public function GetFormID() {
		return $this->db->CreateFormID();
	}

	public function CheckFormID($sha1, $disable=true) {
		$this->db->Begin();
		$form_id = $this->db->GetFormID($sha1);
		var_dump($form_id, $sha1);
		if (!$form_id || $form_id !== $sha1) {
			$GLOBALS['error']='送信済みです';
			$this->db->Commit();
			return false;
		} else {
			$this->db->RemoveFormID($sha1);
			$this->db->Commit();
		}
		return true;
	}
}
