<?php
/*
Copyright (c) 2006 Yasuo Ohgaki All Rights Reserved

This file is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.
 */

// 強制ブラウズ対策
if (!defined('CONFIG')) {
	error_log(
		'直接アクセスを検知しました。 ファイル: '. __FILE__.' 行:'.__LINE__.
		' IP: '. $_SERVER['REMOTE_ADDR'] .' AGENT: '. $_SERVER['HTTP_USER_AGENT']);
	die('このファイルは直接参照できません。');// 確実に実行を終了する
}

define('MAINDB', DATADIR . 'bbs.dat');

abstract class DB {
	protected $db_file; // データベースファイル
	protected $db_parm; // データベースファイルのパーミッション

	protected $db;  // DBオブジェクト（SQLite)

	public function __construct($db_file, $db_parm = 0640)
	{
		$this->db_file = $db_file;
		$this->db_parm = $db_parm;
		$this->open();
	}

	protected function Open()
	{
		$errmsg = null;
		$this->db =  new PDO('sqlite:'.$this->db_file);
		if (!$this->db) {
			trigger_error($errmsg, E_USER_ERROR);
		}
	}

	protected function Close()
	{
		$this->db = null;
	}

	public function Exec($sql)
	{
		return $this->db->exec($sql);
	}

	public function Query($sql)
	{
		return $this->db->query($sql);
	}

	public function Quote($str)
	{
		return $this->db->quote($str);
	}
} // BBS_DB

class BBS_DB_Main extends DB {
	private $secret = 'SH;alncasd;k8792a'; // ハッシュ用 - 秘密の文字列

	public function __construct()
	{
		parent::__construct(MAINDB);
	}

	// スレッドリスト
	public function GetThreads() {
		$sql = "SELECT * FROM thread ORDER BY created DESC;";
		$stmt = $this->Query($sql);
		if (!is_object($stmt)) {
			return false;
		}
		$threads = $stmt->fetchAll(PDO::FETCH_ASSOC);
		return $threads;
	}

	// スレッド作成
	public function CreateThread($title, $message) {
		$sql = "INSERT INTO thread " .
				"(title, message, created) VALUES " .
			"(". $this->quote($title) .", ". $this->quote($message) .", ". $this->quote(date('Y-m-d H:i:s')) .");";
		if (!$this->Exec($sql)) {
			trigger_error('データベースエラーが発生しました');
		}
		return true;
	}

	// スレッドとメッセージ
	public function GetThread($id) {
		$sql = "SELECT * FROM thread WHERE id = ". $this->quote($id) .";";
		$stmt = $this->Query($sql);
		if (!is_object($stmt)) {
			return false;
		}
		$thread = $stmt->fetchAll(PDO::FETCH_ASSOC);
		if ($thread) {
			return $thread[0];
		} else {
			return null;
		}
	}

	//認証
	public function GetUser($user, $pass, $auto_login=false) {
		if ($auto_login) {
			$sql = "SELECT * FROM user WHERE username = ". $this->quote($user) .";";
		} else {
			$sql = "SELECT * FROM user WHERE username = ". $this->quote($user) ." AND  password = ".$this->quote(sha1($pass)) .";";
		}
		$stmt = $this->Query($sql);
		if (!is_object($stmt)) {
			trigger_error('データベースエラーが発生しました');
		}
		$user = $stmt->fetchAll(PDO::FETCH_ASSOC);
		if (isset($user[0])) {
			return $user[0]; //ユーザレコードを返す
		}
		return false;
	}

	// 自動ログインを設定
	public function SetAutoLogin($username) {
		// /dev/urandomなどのセキュアなRNG（乱数生成機)を利用した方が良い
		// $this->secretが安全ならこれでも安全
		$sha1 = sha1($this->secret . mt_rand() . microtime());
		// 全ての自動ログインキーを消す。この自動ログインは複数ブラウザに対応しない。
		$sql = "DELETE FROM autologin WHERE username = ". $this->quote($username) .";";
		$this->Exec($sql);
		$sql = "INSERT INTO autologin (key, username, created) VALUES " .
			"(". $this->quote($sha1) .", ". $this->quote($username) .", ". $this->quote(time()) .")";
		if (!$this->Exec($sql)) {
			trigger_error('自動ログインキーの挿入に失敗しました。');
		}
		// BBS_Authがクッキーを設定できるよう$sha1を返す
		return $sha1;
	}

	public function UnsetAutoLogin($key) {
		$sql = "DELETE FROM autologin WHERE key = ". $this->quote($key) .";";
		$this->Exec($sql);
		return true;
	}

	// 自動ログインを確認
	public function GetAutoLogin($login_key) {
		$sql = "SELECT * FROM autologin WHERE key = ".$this->quote($login_key)." AND created > ". $this->quote(time() - AUTOLOGIN_EXPIRE) .";";
		$stmt = $this->Query($sql);
		if (!is_object($stmt)) {
			return false;
		}
		$rec = $stmt->fetchAll(PDO::FETCH_ASSOC);
		// 掃除 - 最後の掃除時刻を取得
		$sql = "SELECT created FROM autologin ORDER BY created ASC LIMIT 1;";
		$stmt = $this->Query($sql);
		if (!is_object($stmt)) {
			trigger_error('データベースエラーが発生しました');
		}
		$gc_time = $stmt->fetchAll(PDO::FETCH_ASSOC);
		if (!empty($gc_time)) {
			// 1日前のレコードがある場合、1日より古いレコードを削除
			if ($gc_time[0]['created'] < (time() - AUTOLOGIN_EXPIRE)) {
				$sql = "DELETE FROM autologin WHERE created < ". $this->quote(time() - AUTOLOGIN_EXPIRE) .";";
				$stmt = $this->Query($sql);
				if (!is_object($stmt)) {
					trigger_error('データベースエラーが発生しました');
				}
			}
		}

		if ($rec) {
			return $rec[0]['username'];
		} else {
			return null;
		}
	}

	public function Begin() {
		$this->Exec('BEGIN EXCLUSIVE TRANSACTION;');
	}

	public function Commit() {
		$this->Exec('COMMIT TRANSACTION;');
	}

	// FORM_IDを取得
	public function CreateFormID() {
		if (!session_id()) {
			trigger_error('セッションがありません');
		}
		// /dev/urandomなどのセキュアなRNG（乱数生成機)を利用した方が良い
		// $this->secretが安全ならこれでも安全
		$sha1 = sha1($this->secret . mt_rand() . microtime()).'-'.sha1(session_id());
		$sql = "INSERT INTO form_id (sha1, created) VALUES (". $this->quote($sha1).", ". $this->quote(time()) .");";
		if (!$this->Exec($sql)) {
			trigger_error('FORM_IDの作成に失敗しました。');
		}
		return $sha1;
	}

	// FORM_IDを検索
	public function GetFormID($sha1) {
		$tmp = preg_split('/-/', $sha1);
		if ($tmp[1] != sha1(session_id())) {
			trigger_error('セッションIDが無効です。');
		}
		$this->Begin();
		$sql = "SELECT * FROM form_id WHERE sha1 = ". $this->quote($sha1).";";
		$stmt = $this->Query($sql);
		$this->Commit();
		if (!is_object($stmt)) {
			trigger_error('データベースエラーが発生しました');
		}
		$rec = $stmt->fetchAll(PDO::FETCH_ASSOC);
		assert(count($rec) === 1 || count($rec) === 0);
		if (!$rec) {
			return 'no record';
		} else {
			return $rec[0]['sha1'];
		}
	}

	// FORM_IDを削除
	public function RemoveFormID($sha1) {
		$sql = "DELETE FROM form_id WHERE sha1 = ". $this->quote($sha1) .";";
		if (!$this->Exec($sql)) {
			// GCがあれば無い場合もある
			return false;
		}
		return true;
	}
} // BBS_DB


