<?php
/*
Copyright (c) 2006 Yasuo Ohgaki All Rights Reserved

This file is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.
 */

// 強制ブラウズ対策
if (!defined('CONFIG')) {
	error_log(
		'直接アクセスを検知しました。 ファイル: '. __FILE__.' 行:'.__LINE__.
		' IP: '. $_SERVER['REMOTE_ADDR'] .' AGENT: '. $_SERVER['HTTP_USER_AGENT']);
	die('このファイルは直接参照できません。');// 確実に実行を終了する
}

// モード用定数
define('VLD_EXIT', 1);   // エラーがある場合、エラーハンドラでスクリプトの処理は終了する。デフォルト
define('VLD_RETURN', 2); // エラーステータスを返して処理を続行する

// フェイルセーフ設定
define('VLD_FAIL_SAFE', FALSE); // エラーで実行停止しないエラーハンドラ用のフェイルセーフの有効、無効

// mbstring設定 - 5.6未満の古いPHPではEUC-JPがデフォルトになる
mb_regex_encoding('UTF-8');

/*

これらの関数はset_error_handlerのハンドラで全てのエラーで終了すること
を前提としていますvalidate_str_*()はvalidate_str()が既に呼ばれているこ
とを前提としています。問題がある場合はtrue、ない場合falseです。

低レベルなバリデーション関数としてはis_string()/is_array()のチェックが
重複する部分がが今ひとつです。低レベルのバリデーション関数と高レベルの
バリデーション関数に分離するとよりよりすっきりします。

バリデーションは基本ホワイトリストで考え、作ります。正規表現もマッチす
る表現を作る方が簡単です。できる限りホワイトリストで作りますが、ホワイ
トリストの定義が困難、性能的な問題など、適切な理由がある場合はブラック
リストを使っても構いません。正規表現でブラックリストを作る場合、先読み
が必要になるなど正規表現が複雑化します。このためマッチ後に否定する
eregn/ereginがあります。ここでは正規表現のPOSIX文字クラス（ロケールな
どに依存する正規表現もある）を利用せず、改行/タブ以外の制御文字を除外
する場合などにブラックリストを利用しています。

バリデーションクラスにする場合、各関数の最後で$thisを返すと
$vd = new Validator;
$vd->setString('str')->length(1,1000)->noentities();
のような感じで利用できるようになります。

バリデーション関数は定期的に見直しが必要です。例えば、PHP7で議論されて
いる強い型が実装された場合、確実に見直しが必要になります。プロトコルを
省略したURL表記（//example.com/path/to/file）なども最近になって追加さ
れました。

*/

//////////////////////入力チェック////////////////////////
// FALSEの場合にエラー生成のみ
// バリデーションに既存のチェック関数などを利用する場合を想定
function validate_error($bool, $mode=VLD_EXIT) {
	$error = (
		!is_bool($bool)
	);
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('バリデーションエラーを検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 配列のキーがラベルとして、値が文字列として妥当かチェック
function validate_array($array, $mode=VLD_RETURN) {
	// 配列のキーと要素をチェック
	// $_GET/$_POSTなどのキーには多少制約があるが、配列のキーも値もバイナリセーフ
	$tmp = $array;
	$error = false;
	foreach($tmp as $k => $v) {
		if (!validate_str_label($k, VLD_RETURN) == false) {
			$error = true;
			break;
		}
		if (is_scalar($v)
			&& !validate_str($v, VLD_RETURN)) {
			$error = true;
			break;
		} else {
			continue;
		}
		if (is_array($v)) {
			validate_array($v);
		} else {
			trigger_error('サポートされないデータ型がわたされました。'.gettype($v));
			if (VLD_FAIL_SAFE) exit; // フェイルセーフ
		}
	}
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 配列のキーがラベルとして妥当かチェック
function validate_array_key_label($str, $array, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !is_array($array)
		|| !validate_str($str, VLD_RETURN)
	);
	foreach($array as $key=>$val) {
		if (!validate_str_label($key)) {
			$error = true;
			break;
		}
		if (is_array($val)) {
			$error = validate_array_key_label($val);
			if ($error) {
				break;
			}
		}
	}
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 配列のキーが文字列として妥当かチェック
function validate_array_key_str($str, $array, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !validate_str($str, VLD_RETURN)
	);
	foreach($array as $key=>$val) {
		if (!validate_str($key)) {
			$error = true;
			break;
		}
		if (is_array($val)) {
			$error = validate_array_key_str($val);
			if ($error) {
				break;
			}
		}
	}
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 配列の値が文字列として妥当かチェック
function validate_array_val_str($str, $array, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !validate_str($str, VLD_RETURN)
	);
	foreach($array as $val) {
		if (!validate_str($val)) {
			$error = true;
			break;
		}
		if (is_array($val)) {
			$error = validate_array_val_str($val);
			if ($error) {
				break;
			}
		}
	}
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// $arrayのキーが$expected配列に含まれるかチェック
function validate_array_keys_exp($array, $expected, $mode=VLD_EXIT) {
	$error = (
		!is_array($array)
		|| !is_array($expected)
		|| count($array) !== count($expected)
		|| count(array_intersect_key($array, $expected)) != count($expected)
	);
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// $arrayの値が$expected配列に含まれるかチェック
function validate_array_vals_exp($array, $expected, $mode=VLD_EXIT) {
	$error = (
		!is_array($array)
		|| !is_array($expected)
		|| count($array) !== count($expected)
		|| count(array_intersect($array, $expected)) != count($expected)
	);
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な配列を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// $keyが$expected配列に含まれるかチェック
function validate_array_key_in($key, $expected, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($key)
		|| !is_array($expected)
		|| !isset($expected[$key])
	);
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なキーを検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// $valが$expected配列に含まれるかチェック
function validate_array_val_in($val, $expected, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($val)
		|| !is_array($expected)
		|| !in_array($val, $expected)
	);
	if ($mode = VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な値を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// SHA1文字列かチェック
function validate_sha1($str, $mode=VLD_EXIT) {
	// hex形式のSHA1値は0-9, a-fまので40文字。大文字はNG
	$error = (
		!is_scalar($str)
		|| 40 != strspn($str, '1234567890abcdef')
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なSHA1の値を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// MD5文字列かチェック
function validate_md5($str, $mode=VLD_EXIT) {
	// hex形式のMD5値は0-9, a-fまので32文字。大文字はNG
	$error = (
		!is_scalar($str)
		|| 32 != strspn($str, '1234567890abcdef')
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なSHA1の値を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

//メールアドレス形式かチェック
function validate_email($str, $check_dns=true, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		// 正規表現は仕様に合わせて調整
		|| !preg_match ('/\A[a-zA-Z0-9\._-]+@[a-zA-Z0-9\._]+\z/', $str)
	);
	if (!$error) {
		$tmp = preg_split('/@/',$str);
		if (
			count($tmp) > 2
			|| strlen($tmp[0]) > 255
			|| strlen($tmp[1]) > 255
			|| ($check_dns ? !checkdnsrr($tmp[1],'MX') : false) // MX（メールエクエンジャ)レコードがないサーバーは無視
		) {
			$error = true;
		}
	}
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なメール形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

//URL形式かチェック
function validate_url($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		// より長いクエリ文字列が必要な場合は調整
		|| strlen($str) > 2000
		// 正規表現は仕様に合わせて調整 - 特殊文字（特定のコンテクストで意味がある文字）も許可している点に注意
		|| !preg_match('/\A(((https?|ftp|news):\/\/|mailto:)[\w\/\@\$()!?&%#:;.,~\'=*+-]+)\z/', $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なURL形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 整数値形式かチェック
function validate_int($str, $mode=VLD_EXIT) {
	// +-はサポートしていない
	$error = (
		!is_scalar($str)
		|| strlen($str) > 32
		|| strlen($str) != strspn($str, '1234567890')
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な整数形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 整数値が範囲内かチェック
function validate_int_range($str, $min, $max, $mode=VLD_EXIT) {
	// +-はサポートしていない
	$error = (
		!is_scalar($str)
		|| strlen($str) > 32
		|| strlen($str) != strspn($str, '1234567890')
		|| $str < $min || $str > $max
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な整数を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 整数値が整数型範囲内に収まっているかチェック
function validate_int_type($str, $mode=VLD_EXIT) {
	// +-はサポートしていない
	$error = (
		!is_scalar($str)
		|| !is_int($str*1) // 浮動小数点に変換されることを利用
		|| strlen($str) != strspn($str, '1234567890')
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正な整数形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// HEXリテラル形式かチェック
function validate_hex($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| strlen($str)%2
		|| !mb_ereg('\A("0x"[0-9a-fA-F]+)\z', $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なHEX形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 実数形式かチェック
function validate_real($str, $mode=VLD_EXIT) {
	// +-,指数形式はサポートしない。
	$error = (
		!is_scalar($str)
		|| strlen($str) > 32
		|| !mb_ereg('\A(([0-9]*[\.][0-9]+)|([0-9]+[\.][0-9]*))\z', $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error){
		trigger_error('不正な実数形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 実数形式が範囲内かチェック
function validate_real_range($str, $min, $max, $mode=VLD_EXIT) {
	// +-,指数形式はサポートしない。
	$error =  (
		!is_scalar($str)
		|| strlen($str) > 32
		|| !mb_ereg('\A(([0-9]*[\.][0-9]+)|([0-9]+[\.][0-9]*))\z', $str)
		|| $min <= $str
		|| $str <= $max
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error){
		trigger_error('不正な実数を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 数値形式かチェック
function validate_number($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| strlen($str) > 32
		|| !is_numeric($str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error){
		trigger_error('不正な数値形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

//文字列として妥当かチェック
function validate_str($str, $mode=VLD_EXIT) {
	// 長さチェックなし
	$error = (
		!is_scalar($str)
		|| (strlen($str) && !mb_check_encoding($str, 'UTF-8'))
		|| 0 < preg_match('/[\x00-\x09\x0b\x0c\x0e-\x1f\x7f]/u', $str) // 半角スペースとCRとLF以外の制御文字
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

//HEX文字列として妥当かチェック
function validate_str_hex($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| strlen($str)%2
		|| !mb_ereg('\A([0-9a-fA-F]+)\z', $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なHEX形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 文字列がラベル（関数名など）として妥当かチェック
function validate_str_label($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !preg_match('/\A[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*\z/u', $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 正規表現にマッチしなかった場合にエラー
function validate_str_eregn($str, $regex, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !mb_ereg($regex, $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 正規表現にマッチしなかった場合にエラー
function validate_str_eregin($str, $regex, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !mb_eregi($regex, $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 正規表現にマッチした場合にエラー
function validate_str_ereg($str, $regex, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| mb_ereg($regex, $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 正規表現にマッチした場合にエラー
function validate_str_eregi($str, $regex, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| mb_eregi($regex, $str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 余計な文字が含まれていないかチェック
function validate_str_spn($str, $chars, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| strspn($str, $chars) != strlen($str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}


// 余計な文字列が含まれていないかチェック
function validate_str_str($str, $strings, $mode=VLD_EXIT) {
	$error = (
		!validate_str($str, VLD_RETURN)
		|| !strstr($str, $strings)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 文字列の長さをチェック
function validate_str_length($str, $min, $max, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| strlen($str) < $min
		|| strlen($str) > $max
		|| !validate_str($str, VLD_RETURN)
	);
	
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// 文字列の幅をチェック
function validate_str_width($str, $min, $max, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| mb_strwidth($str, 'UTF-8') < $min
		|| mb_strwidth($str, 'UTF-8') > $max
		|| !validate_str($str, VLD_RETURN)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// タグが含まれていないかチェック
function validate_str_notags($str, $mode=VLD_EXIT) {
	$error = (
		!validate_str($str, VLD_RETURN)
		|| strlen(strip_tags($str)) != strlen($str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}

// エンティティ化される文字が含まれていないかチェック
function validate_str_noentities($str, $mode=VLD_EXIT) {
	$error = (
		!is_scalar($str)
		|| !validate_str($str, VLD_RETURN)
		|| strlen(htmlentities($str, ENT_QUOTES, 'UTF-8')) != strlen($str)
	);
	if ($mode == VLD_RETURN) {
		return !$error;
	}
	if ($error) {
		trigger_error('不正なテキスト形式を検出しました。');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
}


///////////////////// 出力 ///////////////////////////
// HTMLエスケープして出力
function html($str) {
	$error = (
		!is_scalar($str)
		|| 0 < preg_match('/[\x00-\x09\x0b\x0c\x0e-\x1f\x7f]/u', $str)
	);
	$ret = htmlspecialchars($str, ENT_QUOTES, 'UTF-8');
	if (!$ret && strlen($str)) {
		$error = true;
	}
	if ($error) {
		trigger_error('入力文字列が不正です');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	}
	return $ret;
}

// HTMLエスケープなしで出力
function html_raw($str) {
	$ret = '';
	$error = (
		!is_scalar($str)
		|| !mb_check_encoding($str, 'UTF-8')
		|| 0 < preg_match('/[\x00-\x09\x0b\x0c\x0e-\x1f\x7f]/u', $str)
	);
	if ($error) {
		trigger_error('入力文字列が不正です');
		if (VLD_FAIL_SAFE) exit; // フェイルセーフ
	} else {
		$ret = $str;
	}
	return $ret;
}

// SQL
function sql($str) {
	trigger_error('PDOになったのでPDOオブジェクトからquoteメソッドでエスケープ');
	if (VLD_FAIL_SAFE) exit; // フェイルセーフ
}

